package com.najva.najvaflutter;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.IntentFilter;

import com.najva.sdk.Najva;
import com.najva.sdk.NajvaClient;
import com.najva.sdk.NajvaConfiguration;
import com.najva.sdk.NajvaJsonDataListener;
import com.najva.sdk.NotificationClickListener;
import com.najva.sdk.NotificationReceiveListener;
import com.najva.sdk.UserSubscriptionListener;


import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;

/**
 * NajvaflutterpluginPlugin
 * created by shayan
 * flutter plugin for implement all features of najva android sdk in flutter
 **/
public class NajvaflutterPlugin implements MethodCallHandler,
        NajvaPluginUserHandler.INajvaPluginUserHandler {

    /**
     * final variables used for method names
     */
    private static final String INIT = "init";
    private static final String GET_SUBSCRIBED_TOKEN = "get_subscribed_token";

    /**
     * final variables for dart method names
     */
    private static final String NEW_JSON_DATA = "onNewJSONData";
    private static final String NEW_USER_SUBSCRIBED = " onUserSubscribed";
    private static final String NEW_NOTIFICATION_RECEIVED = "onNotificationReceived";
    private static final String NEW_NOTIFICATION_CLICKED = "onNotificationClicked";
    private static final String ENABLE_LOCATION = "enable_location";
    private static final String GET_ACTIVITY_JSON = "get_json";

    /**
     * Plugin registration.
     */
    public static void registerWith(PluginRegistry.Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "najvaFlutterplugin");
        channel.setMethodCallHandler(new NajvaflutterPlugin(channel, registrar.context()));

    }

    /* class implementation */

    private MethodChannel channel;
    private Context context;
    private NajvaConfiguration configuration;


    private NajvaflutterPlugin(MethodChannel channel, Context context) {
        this.channel = channel;
        this.context = context;
        configuration = new NajvaConfiguration();
    }


    @Override
    public void onMethodCall(MethodCall call, Result result) {
        switch (call.method) {
            case INIT:
                result.success(init());
                break;
            case GET_SUBSCRIBED_TOKEN:
                result.success(getSubscribedToken());
                break;
            case ENABLE_LOCATION:
                enableLocation();
                result.success(null);
                break;
            case GET_ACTIVITY_JSON:
                result.success(getActivityJson());
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    private String getActivityJson() {
        if (context instanceof Activity) {
            Activity activity = (Activity) context;
            if (activity.getIntent().getExtras() != null)
                return activity.getIntent().getExtras().getString(Najva.NOTIFICATION_JSON);
        }
        return null;
    }

    private void enableLocation() {
        configuration.enableLocation();
    }

    private void handleUsersToken() {
        initUserHandler(new NajvaPluginUserHandler(this));
    }

    private void handleJSONNotification() {
        configuration.setNajvaJsonDataListener(new NajvaJsonDataListener() {
            @Override
            public void onReceiveJson(String s) {
                onJSONDataReceived(s);
            }
        });
    }

    private void initUserHandler(UserSubscriptionListener handler) {
        configuration.setUserSubscriptionListener(handler);
    }

    private boolean init() {
        handleJSONNotification();
        handleUsersToken();

        configuration.setNotificationClickListener(new NotificationClickListener() {
            @Override
            public void onClickNotification(String notificationId, int buttonId) {
                Map<String, String> data = new HashMap<>();
                data.put("notification_id", notificationId);
                data.put("button_id", String.valueOf(buttonId));
                channel.invokeMethod(NEW_NOTIFICATION_CLICKED, data);
            }
        });

        configuration.setReceiveNotificationListener(new NotificationReceiveListener() {
            @Override
            public void onReceiveNotification(String notificationId) {
                Map<String, String> data = new HashMap<>();
                data.put("notification_id", notificationId);
                channel.invokeMethod(NEW_NOTIFICATION_RECEIVED, data);
            }
        });

        Context app = context.getApplicationContext();
        if (app instanceof Application) {
            ((Application) app).registerActivityLifecycleCallbacks(
                    NajvaClient.getInstance(context.getApplicationContext(), configuration)
            );
        }
        return true;
    }

    /**
     * calls method on dart code when new JSON data has been received
     *
     * @param data is JSONObject has received from najva service
     */
    private void onJSONDataReceived(String data) {
        channel.invokeMethod(NEW_JSON_DATA, data);
    }

    @Override
    public void najvaUserSubscribed(String token) {
        channel.invokeMethod(NEW_USER_SUBSCRIBED, token);
    }

    private String getSubscribedToken() {
        return NajvaClient.getInstance().getSubscribedToken();
    }

}